#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  gibareto.py
#  version 0.3
#
#  Copyright 2010 Miguel Ángel Molina <sslapp@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>
#
#   Este programa es software libre: usted puede redistribuirlo y/o modificarlo
#   bajo los términos de la Licencia Pública General GNU publicada
#   por la Fundación para el Software Libre, ya sea la versión 3
#   de la Licencia, o (a su elección) cualquier versión posterior.
#
#   Este programa se distribuye con la esperanza de que sea útil, pero
#   SIN GARANTÍA ALGUNA; ni siquiera la garantía implícita
#   MERCANTIL o de APTITUD PARA UN PROPÓSITO DETERMINADO.
#   Consulte los detalles de la Licencia Pública General GNU para obtener
#   una información más detallada.
#
#   Debería haber recibido una copia de la Licencia Pública General GNU
#   junto a este programa.
#   En caso contrario, consulte <http://www.gnu.org/licenses/>.

import getopt
import gettext
import sys

__version__ = '0.3'

def usage():
  """Show usage instructions."""

  print _("""Gibareto usage:
  gibareto.py [-n|--no-gui] [-h|--help] [-a|--auto]
  -n | --no-gui use console interface
  -h | --help   print usage and exit
  -a | --auto   perform an automatic unattended backup of all accounts
    """)

def start_console():
  """Start console interface."""

  from lib import gb_console
  gb_console.GibaretoApp()

def start_gui():
  """Start GUI interface."""

  from lib import gb_gui
  gb_App = gb_gui.GibaretoApp()
  gb_App.MainLoop()

def main():
  """Gibareto main."""

  # Detect OS platform
  if sys.platform == 'win32':
    import locale
    import os
    loc, encod = locale.getdefaultlocale()
    os.environ['LC_MESSAGES'] = loc
    win32 = True
  else:
    win32 = False

  # Get the translatables strings
  if win32:
    gettext.install('gibareto.win-850', sys.path[0] + '/translations/')
  else:
    gettext.install('gibareto', sys.path[0] + '/translations/')

  # GUI enable by default
  GUI = True

  # Parse command line options
  try:
    opts, args = getopt.getopt(sys.argv[1:], 'nha', ['no-gui', 'help', 'auto'])
  except getopt.GetoptError as err:
    print "Error: {0:s}".format(err.msg)
    usage()
    sys.exit(2)

  for o, a in opts:
    if o in ('-n', '--no-gui'):
      GUI = False
    elif o in ('-h', '--help'):
      usage()
      sys.exit(0)
    elif o in ('-a', '--auto'):
      # TODO
      pass

  # Start gibareto
  try:
    if not GUI:
      # Start console interface
      start_console()
    else:
      # Check if GUI is available
      try:
        import wx
      except ImportError:
        print _("wxPython modules not available\nStarting console interface...")
        # Start console interface
        start_console()
      else:
        # Get the translatables strings
        if win32:
          gettext.install('gibareto.win-1252', sys.path[0] + '/translations/', unicode=True)
        else:
          gettext.install('gibareto', sys.path[0] + '/translations/',unicode=True)
        # Start GUI interface
        start_gui()
  except:
    print _("Unhandled exception: "), sys.exc_info()[1]
    print _("Exiting...")
    sys.exit(-1)

  return 0

if __name__ == '__main__':
  main()

