# -*- coding: utf-8 -*-
#
#  lib.gb_backup.py
#
#  Copyright 2010 Miguel Ángel Molina <sslapp@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>
#
#   Este programa es software libre: usted puede redistribuirlo y/o modificarlo
#   bajo los términos de la Licencia Pública General GNU publicada
#   por la Fundación para el Software Libre, ya sea la versión 3
#   de la Licencia, o (a su elección) cualquier versión posterior.
#
#   Este programa se distribuye con la esperanza de que sea útil, pero
#   SIN GARANTÍA ALGUNA; ni siquiera la garantía implícita
#   MERCANTIL o de APTITUD PARA UN PROPÓSITO DETERMINADO.
#   Consulte los detalles de la Licencia Pública General GNU para obtener
#   una información más detallada.
#
#   Debería haber recibido una copia de la Licencia Pública General GNU
#   junto a este programa.
#   En caso contrario, consulte <http://www.gnu.org/licenses/>.

import cPickle
import os
import time
from lib import gb_imap

IDS_FILE = 'ids.pkl'
MAILBOX_DIR = 'mailbox/'
LAST_RUN_DATE_FILE = 'last-run-date.txt'
IDS = {}

def backup(global_data, user_data, dates_data, main_frame=None):
  """Backup e-mails.

    Params:
    global_data:  mail_host, mail_host_port, backup_path
    user_data:    user_name, password
    dates_data:   date_from: Start date  (format: ddmmyyyy) (default: from first message)
                  date_to:   End date    (format: ddmmyyyy) (default: today)
                  from_last: Boolean     (True: from last run date)
    main_frame: mainFrame instance for GUI
  """

  global IDS

  __mail_host = global_data['mail_host']
  __mail_host_port = global_data['mail_host_port']
  __backup_path = global_data['backup_path']
  __user_name = user_data['user_name']
  __password = user_data['password']
  __date_from = dates_data['date_from']
  __date_to = dates_data['date_to']
  __from_last = dates_data['from_last']

  extra = '/'
  if __backup_path[-1] == '/':
    extra = ''
  BACKUP_PATH = __backup_path + extra + __user_name.split('@')[0]
  MAILBOX_PATH = os.path.join(BACKUP_PATH, MAILBOX_DIR)
  IDS_PATH = os.path.join(BACKUP_PATH, IDS_FILE)
  LAST_RUN_DATE_PATH = os.path.join(BACKUP_PATH, LAST_RUN_DATE_FILE)
  Num2month = {1: 'Jan', 2: 'Feb', 3: 'Mar', 4: 'Apr', 5: 'May', 6: 'Jun',
               7: 'Jul', 8: 'Aug', 9: 'Sep', 10: 'Oct', 11: 'Nov', 12: 'Dec'}

  NOW = time.strftime("%d%m%Y", time.localtime())
  NOW = NOW[0:2] + '-' + Num2month[int(NOW[2:4])] + '-' + NOW[4:8]

  start_date = __date_from
  end_date = __date_to

  # Check if mailbox path exists and create it if not
  if not os.path.isdir(MAILBOX_PATH):
    try:
      os.makedirs(MAILBOX_PATH, 0700)
    except OSError as error:
      if main_frame != None:
        main_frame.parent.statusBar1.SetStatusText(error.strerror + ' : ' + error.filename)
      else:
        print error.strerror, error.filename
      return -1

  # Check params
  if __from_last:
    if os.path.isfile(LAST_RUN_DATE_PATH):
      try:
        fd = open(LAST_RUN_DATE_PATH, 'r')
        # Read the last run date from file
        start_date = fd.read(11)
        end_date = ''
        fd.close()
      except IOError as error:
        if main_frame != None:
          main_frame.parent.statusBar1.SetStatusText(error.strerror + ' : ' + error.filename)
        else:
          print error.strerror, error.filename
        start_date = ''
        end_date = ''
    else:
      start_date = ''
      end_date = ''
  else:
    if __date_from != '':
      try:
        start_date = __date_from[0:2] + '-' + Num2month[int(__date_from[2:4])] + '-' + __date_from[4:8]
      except BaseException as error:
        if main_frame != None:
          main_frame.parent.statusBar1.SetStatusText(_("Date from error.") + ' : ' + error)
        else:
          print _("Date from error."), error
        start_date = ''

    if __date_to != '':
      try:
        end_date = __date_to[0:2] + '-' + Num2month[int(__date_to[2:4])] + '-' + __date_to[4:8]
      except BaseException as error:
        if main_frame != None:
          main_frame.parent.statusBar1.SetStatusText(_("Date to error.") + ' : ' + error)
        else:
          print _("Date to error."), error
        end_date = ''

  # Load IDs if file exists
  if os.path.isfile(IDS_PATH):
    try:
      fd = open(IDS_PATH, 'rb')
      IDS = cPickle.load(fd)
      fd.close()
    except (IOError, PickleError) as error:
      if main_frame != None:
        main_frame.parent.statusBar1.SetStatusText(_("Error loading IDS file.") + ' : ' + error)
      else:
        print _("Error loading IDS file."), error

  try:
    # Create the mail server connection and login into the defined account
    ms = gb_imap.MailServer(__mail_host, __mail_host_port, __user_name, __password)
    ms.login()
    # Get the mailboxes
    ms.get_mailboxes()
    # Loop through the mailboxes
    mbk = ms.mailboxes.keys()
    mbk.sort()
    mbk_len = len(mbk)
    mbk_pos = 1
    for k in mbk:
      if main_frame != None:
        main_frame.progress1.SetValue(mbk_pos * 100 / mbk_len)
        main_frame.progress1.Update()
        main_frame.parent.statusBar1.SetStatusText(_('{0:02d}% completed...').format(mbk_pos * 100 / mbk_len),1)

      mb = ms.mailboxes[k]
      label = mb.name
      msg_list = mb.get_message_list(start_date, end_date)
      msg_list_len = len(msg_list)
      msg_list_pos = 1
      if main_frame == None:
        print _("Mailbox: {0:25s} {1:>5d} messages.").format(label, len(msg_list))
      else:
        main_frame.logText.AppendText(_("Mailbox: {0:25s} {1:>5d} messages.\n").format(label, len(msg_list)))
        main_frame.logText.Update()
      for msg in msg_list:
        if main_frame != None:
          main_frame.progress1.SetValue(msg_list_pos * 100 / msg_list_len)
          main_frame.progress1.Update()
        id = mb.get_message_id(msg)
        if id not in IDS:
          if main_frame == None:
            print "\t"+_("Message")+": {0:s}".format(id)
          else:
            main_frame.logText.AppendText("\t"+_("Message")+": {0:s}\n".format(id))
            main_frame.logText.Update()
          e_mail = mb.get_message(msg, id)
          IDS[id] = [label]
          message_path = os.path.join(MAILBOX_PATH,mb.sub_name)
          # Check if message path exists and create it if not
          if not os.path.isdir(message_path):
            try:
              os.makedirs(message_path, 0700)
            except OSError as error:
              if main_frame != None:
                main_frame.parent.statusBar1.SetStatusText(error.strerror + ' : ' + error.filename)
              else:
                print error.strerror, error.filename
          filename = os.path.join(message_path, id+'.eml')
          try:
            fd = open(filename, 'wb')
            fd.write(e_mail.e_mail.as_string(True))
            fd.close()
          except IOError as error:
            if main_frame != None:
              main_frame.parent.statusBar1.SetStatusText(_("Error writing e-mail file.") + ' : ' + error.strerror + ' : ' + error.filename)
            else:
              print _("Error writing e-mail file."), error.strerror, error.filename
        else:
          if (label not in IDS[id]):
            if main_frame == None:
              print _("Updating label: {0:s}").format(label)
            else:
              main_frame.logText.AppendText(_("Updating label: {0:s}\n").format(label))
              main_frame.logText.Update()
            IDS[id].append(label)
        msg_list_pos = msg_list_pos + 1
      mbk_pos = mbk_pos + 1
    ms.logout()
  except BaseException as error:
    if main_frame == None:
      print "Error: {0:s}".format(error)
      raw_input(_("Press any key..."))
    else:
      main_frame.logText.AppendText("Error: {0:s}\n".format(error))
      main_frame.logText.Update()
    return -1
  try:
    fd = open(IDS_PATH, 'wb')
    cPickle.dump(IDS, fd, -1)
    fd.close()
  except (IOError, PickleError) as error:
    if main_frame != None:
      main_frame.parent.statusBar1.SetStatusText(_("Error writing IDS file.") + ' : ' + error)
    else:
      print _("Error writing IDS file."), error
  try:
    fd = open(LAST_RUN_DATE_PATH, 'wb')
    fd.write(NOW)
    fd.close()
  except IOError as error:
    if main_frame != None:
      main_frame.parent.statusBar1.SetStatusText(_("Error writing file.") + ' : ' + error.strerror + ' : ' + error.filename)
    else:
      print _("Error writing file."), error.strerror, error.filename

