# -*- coding: utf-8 -*-
#
#  lib.gb_console.py
#
#  Copyright 2010 Miguel Ángel Molina <sslapp@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>
#
#   Este programa es software libre: usted puede redistribuirlo y/o modificarlo
#   bajo los términos de la Licencia Pública General GNU publicada
#   por la Fundación para el Software Libre, ya sea la versión 3
#   de la Licencia, o (a su elección) cualquier versión posterior.
#
#   Este programa se distribuye con la esperanza de que sea útil, pero
#   SIN GARANTÍA ALGUNA; ni siquiera la garantía implícita
#   MERCANTIL o de APTITUD PARA UN PROPÓSITO DETERMINADO.
#   Consulte los detalles de la Licencia Pública General GNU para obtener
#   una información más detallada.
#
#   Debería haber recibido una copia de la Licencia Pública General GNU
#   junto a este programa.
#   En caso contrario, consulte <http://www.gnu.org/licenses/>.

import base64
import getpass
from lib import gb_backup
from lib import gb_config

# Internal variables
__config_handler = None
__current_account = ''
__current_from = ''
__current_to = ''
__current_from_last = True

def menu():
  """Show the options menu.

  Returns a valid option string
  """

  options = {_("s"):_("Select account"),
            _("c"):_("Create account"),
            _("m"):_("Modify account"),
            _("d"):_("Delete account"),
            _("b"):_("Backup account"),
            _("r"):_("Restore account"),
            _("f"):_("Configure gibareto"),
            _("q"):_("Quit")}

  # Required to keep options order
  options_list = (_("s"), _("c"), _("m"), _("d"), _("b"), _("r"), _("f"), _("q"))
  while True:
    show_settings()
    print _("Options:")
    for o in options_list:
      print "\t{0:s}: {1:s}".format(o, options[o])
    print

    option = raw_input(_("Choose an option: "))
    if option.isalpha() and option.lower() in options_list:
      return option.lower()
    else:
      print _("Invalid option!")
      raw_input(_("Press any key..."))

def show_settings():
  """Show actual settings."""

  global __current_account, __current_from, __current_to, __current_from_last
  print
  print _("Current account: {0:s}").format(__current_account)
  print _("Dates: from = {0:s} to = {1:s}").format(__current_from, __current_to)
  print _("From Last = "),
  if __current_from_last:
    print _("Enabled")
  else:
    print _("Disabled")
  print

def GibaretoApp():
  """Gibareto console interface."""

  global __config_handler, __current_account, __current_from, __current_to, __current_from_last
  __config_handler = gb_config.ConfigHandler()
  __current_account = __config_handler.get_def_account()['def_account']

  # Main loop
  while True:
    __dates = __config_handler.get_dates()
    __current_from = __dates['date_from']
    __current_to = __dates['date_to']
    __current_from_last = __config_handler.get_from_last()['from_last']
    # Show menu
    option = menu()
    # Quit
    if option == _("q"):
      break
    # Select account
    elif option == _("s"):
      __current_account = select_account(__config_handler, __current_account)
    # Create account
    elif option == _("c"):
      result = __config_handler.edit_account()
      if result != '':
        __current_account = result
      __config_handler.save_config()
    # Modify account
    elif option == _("m"):
      if __current_account == '':
        print _("No account selected.")
        raw_input(_("Press any key..."))
      else:
        __account = __config_handler.get_account(__current_account)
        result = __config_handler.edit_account(__account['user_name'],
                                               __account['password'])
        if result != '':
          __current_account = result
        __config_handler.save_config()
    # Delete account
    elif option == _("d"):
      result = ''
      if __config_handler.config.has_section(__current_account):
        ans = raw_input(_("Are you sure that you want to delete account: ") + __current_account + _(" ? [y/N]")).upper()
        if ans == _('Y'):
          result = __config_handler.delete_account(__current_account)
          print __current_account, _("account deleted")
      if result == 'default_deleted':
        __current_account = ''
    # Backup account
    elif option == _("b"):
      if __current_account == '':
        print _("No account selected.")
        raw_input(_("Press any key..."))
      else:
        global_data = __config_handler.get_mail_host()
        global_data.update(__config_handler.get_backup_path())
        user_data = __config_handler.get_account(__current_account)
        dates_data = __config_handler.get_dates()
        dates_data.update(__config_handler.get_from_last())
        if user_data['password'] == '':
          user_data['password'] = base64.b64encode(getpass.getpass(_("Enter your account password: ")))
        print _('Backup in progress...')
        gb_backup.backup(global_data, user_data, dates_data)
        print _('Backup done!')
    # Restore account
    elif option == _("r"):
      print _("Sorry! Option not implemented!")
    # Configure gibareto
    elif option == _("f"):
      global_data = __config_handler.get_mail_host()
      global_data.update(__config_handler.get_backup_path())
      __config_handler.edit_global_data(global_data['mail_host'],
                                      global_data['mail_host_port'],
                                      global_data['backup_path'])
      __config_handler.edit_dates(__current_from, __current_to)
      __config_handler.edit_from_last()
      __config_handler.save_config()

def select_account(config_handler, current_account):
  """Select an account from the account list."""

  # Return current account if nothing selected
  account = current_account
  print _("\nAccount list:")
  accounts = config_handler.get_accounts()
  l_accounts = len(accounts)
  if l_accounts > 0:
    for a in accounts:
      print "\t{0:>2d}\t{1:s}".format(accounts.index(a), a)
    print
    while True:
      sel = raw_input(_("Select account number: "))
      if sel.isdigit() and int(sel) in range(l_accounts):
        account = accounts[int(sel)]
        break
      elif (sel == ''):
        break
  else:
    print "No accounts defined!"
    raw_input(_("Press any key..."))

  return account
