# -*- coding: utf-8 -*-
#
#  lib.gb_gui.py
#  lib.gb_wx_gui.py
#
#  Copyright 2010 Miguel Ángel Molina <sslapp@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>
#
#   Este programa es software libre: usted puede redistribuirlo y/o modificarlo
#   bajo los términos de la Licencia Pública General GNU publicada
#   por la Fundación para el Software Libre, ya sea la versión 3
#   de la Licencia, o (a su elección) cualquier versión posterior.
#
#   Este programa se distribuye con la esperanza de que sea útil, pero
#   SIN GARANTÍA ALGUNA; ni siquiera la garantía implícita
#   MERCANTIL o de APTITUD PARA UN PROPÓSITO DETERMINADO.
#   Consulte los detalles de la Licencia Pública General GNU para obtener
#   una información más detallada.
#
#   Debería haber recibido una copia de la Licencia Pública General GNU
#   junto a este programa.
#   En caso contrario, consulte <http://www.gnu.org/licenses/>.
import base64
import wx
import wx.lib.wordwrap
from lib import gb_wx_gui as gui
from lib import gb_backup
from lib import gb_config
import sys

class Frame1(gui.Frame1):
  """Main Frame."""

  def __init__(self, parent):
    gui.Frame1.__init__(self, parent)
    self.statusBar1.SetStatusText(_('Application started...'), 0)

  def OnIdle( self, event ):
    self.statusBar1.SetStatusText(_('Idle...'),0)

class Panel1(gui.Panel1):
  """Main Panel."""

  def __init__(self, parent):
    gui.Panel1.__init__(self, parent)
    exec_path = sys.path[0]
    self.logo1.SetBitmap(wx.Bitmap( exec_path + "/icons/gibareto-64x64.ico", wx.BITMAP_TYPE_ANY ))
    self.backupButton.SetBitmapLabel(wx.Bitmap( exec_path + "/icons/backup_icon.png", wx.BITMAP_TYPE_ANY ))
    self.restoreButton.SetBitmapLabel(wx.Bitmap( exec_path + "/icons/restore_icon.png", wx.BITMAP_TYPE_ANY ))
    self.configButton.SetBitmapLabel(wx.Bitmap( exec_path + "/icons/config_icon.png", wx.BITMAP_TYPE_ANY ))
    self.backupButton.SetToolTipString( _("Backup selected account") )
    self.restoreButton.SetToolTipString( _("Restore selected account") )
    self.configButton.SetToolTipString( _("Configure Gibareto") )
    self.parent = parent

    # Read config
    self.config_handler = gb_config.ConfigHandler(self.parent)
    accounts = self.config_handler.get_accounts()
    self.accountsCombo1.SetItems(accounts)
    self.accountsCombo1.SetValue(self.config_handler.get_def_account()['def_account'])
    text_font = self.config_handler.get_text_font()['text_font']
    if text_font != '':
      text_font_family, text_font_face, text_font_style, text_font_pointsize, text_font_weight = text_font.split(':')
      self.logText.SetFont(wx.Font(int(text_font_pointsize), int(text_font_family), int(text_font_style),
                          int(text_font_weight), faceName=text_font_face))
    text_color = self.config_handler.get_text_color()['text_color']
    if text_color != '':
      text_red, text_green, text_blue = text_color.split(':')
      self.logText.SetForegroundColour(wx.Colour(int(text_red), int(text_green), int(text_blue)))
    self.parent.statusBar1.SetStatusText(_('Configuration read.'))

  def OnBackup( self, event ):
    current_account = self.accountsCombo1.GetValue()
    if current_account == None or current_account == '':
      self.logText.AppendText(_("No account selected.\n"))
      self.logText.Update()
    else:
      global_data = self.config_handler.get_mail_host()
      global_data.update(self.config_handler.get_backup_path())
      user_data = self.config_handler.get_account(current_account)
      dates_data = self.config_handler.get_dates()
      dates_data.update(self.config_handler.get_from_last())
      if user_data['password'] == '':
        self.the_password = ''
        PassDialog(self)
        user_data['password'] = self.the_password
      self.parent.statusBar1.SetStatusText(_('Backup in progress...'))
      gb_backup.backup(global_data, user_data, dates_data, self)
      self.parent.statusBar1.SetStatusText(_('Backup done!'))
      self.parent.statusBar1.SetStatusText('',1)
      self.progress1.SetValue(0)

  def OnRestore( self, event ):
    wx.MessageBox(_('Restore not implemented yet!'), _('Sorry'), style=wx.CENTRE, parent=self)

  def OnConfig( self, event ):
    config = ConfigDialog(self, self.config_handler)
    config.ShowModal()
    config.Destroy()
    accounts = self.config_handler.get_accounts()
    self.accountsCombo1.SetItems(accounts)
    self.accountsCombo1.SetValue(self.config_handler.get_def_account()['def_account'])

  def OnAbout( self, event ):
    description = _("""Gibareto is a python application which performs backups of
 GMail accounts into a local folder.
 (The restore feature has not been implemented yet (but soon :-))
      """).replace('\n','')

    licence = _("""This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, see <http://www.gnu.org/licenses/>
      """).replace('\n','')

    info = wx.AboutDialogInfo()

    info.SetName('Gibareto')
    info.SetVersion('0.3')
    info.SetDescription(wx.lib.wordwrap.wordwrap(description, 400, wx.ClientDC(self)))
    info.SetCopyright(u'Copyright 2010 Miguel Ángel Molina <sslapp@gmail.com>')
    info.AddDeveloper(u'Miguel Ángel Molina')
    info.AddDocWriter(u'Miguel Ángel Molina')
    info.AddArtist(u'Miguel Ángel Molina')
    info.AddTranslator(u'Miguel Ángel Molina')
    if sys.platform.find('linux') != -1:
      info.SetWebSite(('https://launchpad.net/gibareto/', _('Gibareto WebSite')))
      info.SetLicence(wx.lib.wordwrap.wordwrap(licence, 400, wx.ClientDC(self)))

    wx.AboutBox(info)

  def OnQuit( self, event ):
    self.parent.Close()

class ConfigDialog(gui.Dialog1):
  def __init__(self, parent, config_handler):
    gui.Dialog1.__init__(self, parent)
    self.parent = parent
    self.config_handler = config_handler
    self.readConfig()

  def readConfig(self):
    # Read global configuration parameters
    current_account = self.config_handler.get_def_account()['def_account']
    dates = self.config_handler.get_dates()
    current_from = dates['date_from']
    current_to = dates['date_to']
    current_from_last = self.config_handler.get_from_last()['from_last']
    mail_host = self.config_handler.get_mail_host()
    self.mail_host.SetValue(mail_host['mail_host'])
    self.mail_host_port.SetValue(mail_host['mail_host_port'])
    self.backup_path.SetValue(self.config_handler.get_backup_path()['backup_path'])
    dt = wx.DateTime()
    if current_from != '':
      dt.Set(int(current_from[:2]), month=int(current_from[2:4])-1, year=int(current_from[4:]))
    self.date_from.SetValue(dt)
    dt = wx.DateTime()
    if current_to != '':
      dt.Set(int(current_to[:2]), month=int(current_to[2:4])-1, year=int(current_to[4:]))
    self.date_to.SetValue(dt)
    self.from_last.SetValue(bool(current_from_last))
    text_font = self.config_handler.get_text_font()['text_font']
    if text_font != '':
      text_font_family, text_font_face, text_font_style, text_font_pointsize, text_font_weight = text_font.split(':')
      self.logFont.SetSelectedFont(wx.Font(int(text_font_pointsize), int(text_font_family), int(text_font_style), int(text_font_weight), faceName=text_font_face))
    text_color = self.config_handler.get_text_color()['text_color']
    if text_color != '':
      text_red, text_green, text_blue = text_color.split(':')
      self.logColor.SetColour(wx.Colour(int(text_red), int(text_green), int(text_blue)))

    # Read accounts
    accounts = self.config_handler.get_accounts()
    self.accountsList.Set(accounts)

  def OnFont( self, event ):
    self.parent.logText.SetFont(event.GetFont())

  def OnColor( self, event ):
    self.parent.logText.SetForegroundColour(event.GetColour())

  def OnAdd( self, event ):
    user_name = self.user_name.GetValue()
    if user_name != '':
      user_name_name = user_name.split('@')[0]
      password = ''
      ans = wx.MessageDialog(None, _('Save encrypted password to config file?'), _('Save password'),
              wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)
      if ans.ShowModal() == wx.ID_YES:
        password = base64.b64encode(self.password.GetValue())
      ans = wx.MessageDialog(None, _('Make this the default account?'), _('Make default'),
              wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)
      if ans.ShowModal() == wx.ID_YES:
        self.config_handler.set_def_account(user_name_name)
      self.config_handler.set_account(user_name, password)
      self.config_handler.save_config()
      self.accountsList.Append(user_name_name)

  def OnDelete( self, event ):
    account = self.accountsList.GetSelection()
    account_name = self.accountsList.GetStringSelection()
    if account != wx.NOT_FOUND:
      ans = wx.MessageDialog(self, _('Are you sure to delete account: ') + account_name + ' ?',
        _('Delete account'), wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)

      if ans.ShowModal() == wx.ID_YES:
        self.config_handler.delete_account(account_name)
        self.config_handler.save_config()
        self.accountsList.Delete(account)
        self.parent.parent.statusBar1.SetStatusText(account_name + ' '+ _('account deleted'))

  def OnSave( self, event ):
    self.config_handler.set_mail_host(self.mail_host.GetValue(),
                                        self.mail_host_port.GetValue())
    self.config_handler.set_backup_path(self.backup_path.GetValue())
    date_from = self.date_from.GetValue()
    if date_from.IsValid():
      date_from = date_from.FormatISODate()
      date_year, date_month, date_day = date_from.split('-')
      date_from = date_day + date_month + date_year
    else:
      date_from = ''
    date_to = self.date_to.GetValue()
    if date_to.IsValid():
      date_to = date_to.FormatISODate()
      date_year, date_month, date_day = date_to.split('-')
      date_to = date_day + date_month + date_year
    else:
      date_to = ''
    self.config_handler.set_dates(date_from, date_to)
    self.config_handler.set_from_last(self.from_last.GetValue())
    text_font = self.logFont.GetSelectedFont()
    text_font_family = str(text_font.GetFamily())
    text_font_face = text_font.GetFaceName()
    text_font_style = str(text_font.GetStyle())
    text_font_pointsize = str(text_font.GetPointSize())
    text_font_weight = str(text_font.GetWeight())
    text_font = text_font_family + ':' + text_font_face + ':' + text_font_style + ':' + text_font_pointsize + ':' + text_font_weight
    self.config_handler.set_text_font(text_font)
    text_color = self.logColor.GetColour()
    text_red = str(text_color.Red())
    text_green = str(text_color.Green())
    text_blue = str(text_color.Blue())
    text_color = text_red + ':' + text_green + ':' + text_blue
    self.config_handler.set_text_color(text_color)
    self.config_handler.save_config()
    self.Close()
    self.parent.parent.statusBar1.SetStatusText(_('Configuration Saved...'))
    self.Close()

  def OnClose( self, event ):
    self.Close()

class PassDialog(gui.PassDialog):
  def __init__(self, parent):
    gui.PassDialog.__init__(self, parent)
    self.parent = parent
    self.password.SetFocus()
    self.ShowModal()

  def OnOk( self, event ):
    self.parent.the_password = base64.b64encode(self.password.GetValue())
    self.Close()

class GibaretoApp(wx.App):

    def OnInit(self):
        frame = Frame1(None)
        panel1 = Panel1(frame)
        frame.SetIcon(wx.Icon(sys.path[0] +  "/icons/gibareto-64x64.ico", wx.BITMAP_TYPE_ICO))
        frame.Show()
        return True
